// Copyright (c) Mito
import React, { Component } from 'react';
import { functionDocumentationObjects } from '../data/function_documentation';
import { fromAgGridColIDtoMitoColumnID } from '../utils/gridData';
import { endsInColumnHeader, getColumnHeaderSuggestions, getFunctionSuggestions } from '../utils/suggestions';
import DocumentationBox from './editor/DocumentationBox';
import SuggestionBox from './editor/SuggestionBox';
// Indicator for the type of editing mode we are in
export var CellEditingMode;
(function (CellEditingMode) {
    CellEditingMode["OFF"] = "off";
    CellEditingMode["COLUMN_EDITING"] = "column_editing";
    CellEditingMode["SINGLE_CELL_EDITING"] = "single_cell_editing";
})(CellEditingMode || (CellEditingMode = {}));
const keyboardKeys = {
    ENTER_KEY: "Enter",
    TAB_KEY: "Tab",
    ESCAPE_KEY: "Escape",
    UP: 'ArrowUp',
    DOWN: 'ArrowDown',
    LEFT: 'ArrowLeft',
    RIGHT: 'ArrowRight',
    // These events have IE/Edge differences
    IE_UP: 'Up',
    IE_DOWN: 'Down',
    IE_LEFT: 'Left',
    IE_RIGHT: 'Right'
};
export const TAKE_SUGGESTION_KEYS = [keyboardKeys.ENTER_KEY, keyboardKeys.TAB_KEY];
const CLOSE_INPUT_KEYS = [keyboardKeys.ENTER_KEY, keyboardKeys.TAB_KEY, keyboardKeys.ESCAPE_KEY];
export const ARROW_UP_KEYS = [keyboardKeys.UP, keyboardKeys.IE_UP];
export const ARROW_DOWN_KEYS = [keyboardKeys.DOWN, keyboardKeys.IE_DOWN];
export const ARROW_LEFT_KEYS = [keyboardKeys.LEFT, keyboardKeys.IE_LEFT];
export const ARROW_RIGHT_KEYS = [keyboardKeys.RIGHT, keyboardKeys.IE_RIGHT];
export default class MitoCellEditor extends Component {
    constructor(props) {
        super(props);
        /*
            We turn editing mode on first to ensure that any future calls to
            set editing formula set the formula of the correct column.
        */
        const columnID = fromAgGridColIDtoMitoColumnID(props.colDef.colId);
        const mode = props.isFormulaColumn ? CellEditingMode.COLUMN_EDITING : CellEditingMode.SINGLE_CELL_EDITING;
        props.setEditingMode(mode, columnID, props.rowIndex);
        /*
            Get the proper formula to display in the cell editor.
            - If the cell editor was opened by pressing a character, then the character should be the formula
            - Othwerise if the cell editor is in COLUMN_EDITING mode, then the formula passed as a prop should be the formula
            - Otherwise we are in SINGLE_CELL_EDITING mode and if the sheetJSON is defined, we should get the cell's value
            - Otherwise, we don't have a formula to display
        */
        const formula = props.charPress != null ? props.charPress : mode === CellEditingMode.COLUMN_EDITING ?
            props.formula : props.sheetJSON !== undefined ?
            // Note we set the value below to a string, as this is what we need 
            // to send to the backend
            '' + props.sheetJSON.data[props.rowIndex][props.sheetJSON.columnIDs.indexOf(columnID)] : '';
        if (props.charPress != null) {
            // When the cell editor is entered through a charPress, we overwrite the formula to the character pressed
            props.setEditingFormula(props.charPress);
        }
        else {
            // Otherwise, we set the editing formula to the starting formula, so that if the user
            // immediately clicks off the cell, the correct value is submitted
            props.setEditingFormula(formula);
        }
        // Set the editor's state
        this.state = {
            mode: mode,
            formula: formula,
            input: null,
            suggestionIndex: 0
        };
        this.getValue = this.getValue.bind(this);
        this.isPopup = this.isPopup.bind(this);
        this.handleOnChange = this.handleOnChange.bind(this);
        this.afterGuiAttached = this.afterGuiAttached.bind(this);
        this.getCurrentSuggestions = this.getCurrentSuggestions.bind(this);
        this.getDocumentationBoxFunction = this.getDocumentationBoxFunction.bind(this);
        this.onKeyDown = this.onKeyDown.bind(this);
        this.onClick = this.onClick.bind(this);
        this.onMouseEnterSuggestion = this.onMouseEnterSuggestion.bind(this);
        this.selectSuggestion = this.selectSuggestion.bind(this);
        this.getAdjustedIndex = this.getAdjustedIndex.bind(this);
    }
    getValue() {
        return this.state.formula;
    }
    /*
        set the cursor location so that if the user uses mouse to reference
        column, we append the column to the correct location
    */
    // eslint-disable-next-line @typescript-eslint/no-inferrable-types
    updateCursorIndex(adjustment) {
        var _a, _b;
        if ((_a = this.state.input) === null || _a === void 0 ? void 0 : _a.selectionEnd) {
            this.props.setCursorIndex(((_b = this.state.input) === null || _b === void 0 ? void 0 : _b.selectionEnd) + adjustment);
        }
    }
    /* Make the cell editor a popup, so we can display suggestion/documentation box */
    isPopup() {
        return true;
    }
    /* Update the cell value while typing */
    handleOnChange(event) {
        this.updateFormulaState(event.target.value);
        this.updateCursorIndex(0);
    }
    /*
        This function updates the state of both the mitoCelEditor formula and the formulaBar
        of in Mito.tsx. We update in both locations so that the formula can persist across
        opening and closing the cell editor
    */
    updateFormulaState(newFormula) {
        this.setState({
            formula: newFormula
        }, () => {
            this.props.setEditingFormula(this.state.formula);
        });
    }
    /*
        This function is called by ag-grid after this component
        is rendered; we simply focus on the input feild rendered
        below so the user can begin typing immediately!

        NOTE: There is a bug that requires us to also call focus
        _after_ we define the ref in the setState callback. For some reason,
        the ref is not always defined after the gui is attached - and so
        we need to focus then as well!
    */
    afterGuiAttached() {
        var _a, _b;
        (_a = this.state.input) === null || _a === void 0 ? void 0 : _a.focus();
        // when the cell editor loads, set the Cursor (cursor) to be in the last position it was in
        // if the cell editor was just closed
        (_b = this.state.input) === null || _b === void 0 ? void 0 : _b.setSelectionRange(this.props.cursorIndex, this.props.cursorIndex);
    }
    /* when the user clicks to move the Cursor, update the Cursor location*/
    onClick() {
        this.updateCursorIndex(0);
    }
    /*
        This function handles key presses on the cell editor input.

        If the suggestion box is open currently, than this function first checks
        if this key press effects the suggestion box (e.g. scrolling or selecting a suggestion).

        Otherwise, we check if this event closes the cell editor.

        NOTE: we event.preventDefault() to stop events from having unintended
        consequences if they cause effects we handle ourselves. For example, this
        prevents TAB from selecting the next form field.

        NOTE: the events handled by this functions should correspond to the events
        suppressed in editing mode in the ag-grid, to avoid unintended behavior
    */
    onKeyDown(event) {
        /*
            If the arrow keys are used to move the Cursor location, update the cursor location

            Note: Idk why, but when moving the cursor with the arrow keys, there is a one index lag
            in the direction the cursor is moving. ie: if moving to the left, the index will be 1 higher than
            it should be. We work around this by using the adjustments passed as params to the updateCursorIndex function below.
        */
        if (ARROW_LEFT_KEYS.includes(event.key)) {
            this.updateCursorIndex(-1);
        }
        else if (ARROW_RIGHT_KEYS.includes(event.key)) {
            this.updateCursorIndex(1);
        }
        const suggestions = this.getCurrentSuggestions();
        if (suggestions !== undefined) {
            if (TAKE_SUGGESTION_KEYS.includes(event.key)) {
                event.preventDefault();
                // We mod below just as one last safety check for an out of bound index, which we
                // also ensure with the getAdjustedIndex function
                const selectedSuggestion = suggestions[this.state.suggestionIndex % suggestions.length];
                return this.selectSuggestion(selectedSuggestion.type, selectedSuggestion.suggestion, selectedSuggestion.match);
            }
            else if (ARROW_UP_KEYS.includes(event.key)) {
                event.preventDefault();
                return this.setState((prevState) => {
                    return { suggestionIndex: this.getAdjustedIndex(prevState.suggestionIndex - 1, suggestions) };
                });
            }
            else if (ARROW_DOWN_KEYS.includes(event.key)) {
                event.preventDefault();
                return this.setState((prevState) => {
                    return { suggestionIndex: this.getAdjustedIndex(prevState.suggestionIndex + 1, suggestions) };
                });
            }
        }
        // NOTE: we should always check the close events _last_, in case the keypresses
        // were trying to do other things!
        if (CLOSE_INPUT_KEYS.includes(event.key)) {
            event.preventDefault();
            this.props.setEditingMode(CellEditingMode.OFF, fromAgGridColIDtoMitoColumnID(this.props.column.getColId()), this.props.rowIndex);
        }
    }
    /*
        This function returns the current suggestions, which are:
        2. Any column who is prepended by the current ending string, which may not be all alphabetic
        characters (and can be any valid column).
        1. Any functions who are prepended by the current ending string, which
        must be all alphabetic characters. Case-insensitive.
        
        Returns undefined if there are no suggestions, or if the end of the formula
        is a perfect column match - so the user can submit in this case!
    */
    getCurrentSuggestions() {
        // If the editor is in SINGLE_CELL_EDITING mode, then don't get any suggestions
        if (this.state.mode === CellEditingMode.SINGLE_CELL_EDITING) {
            return undefined;
        }
        if (typeof this.state.formula === 'string') {
            // If you have a column header that is matched exactly, we _do show no suggestions_, as 
            // we want you to be able to enter it in
            if (endsInColumnHeader(this.state.formula, Object.values(this.props.columnIDsMap))) {
                return undefined;
            }
            const columnHeaderSuggestions = getColumnHeaderSuggestions(this.state.formula, Object.values(this.props.columnIDsMap));
            const functionSuggestions = getFunctionSuggestions(this.state.formula);
            const suggestions = [];
            // Add the column headers first, then the functions
            if (columnHeaderSuggestions !== undefined) {
                suggestions.push(...columnHeaderSuggestions);
            }
            if (functionSuggestions !== undefined) {
                suggestions.push(...functionSuggestions);
            }
            if (suggestions.length > 0) {
                return suggestions;
            }
        }
        return undefined;
    }
    /*
        Keeps the selected index for the suggestion box inbounds; the index
        should be the index of one of the suggestions!
    */
    getAdjustedIndex(index, suggestions) {
        // Fancy expression to keep it as a valid index
        return Math.min(Math.max(0, index), suggestions.length - 1);
    }
    /*
        This function returns the current function that should
        be displayed in the documentation box, based on if the
        documentation box open condition is met.

        If the documentation box open condition is not met, this
        returns undefined.
    */
    getDocumentationBoxFunction() {
        var _a;
        // If the editor is in SINGLE_CELL_EDITING mode, then don't get any documentation
        if (this.state.mode === CellEditingMode.SINGLE_CELL_EDITING) {
            return undefined;
        }
        // Finds all instances of functions that are not followed by a closing paren
        // e.g. all functions that are still being edited.
        const docBoxRe = /[A-Za-z]+\((?![^)]*\))/g;
        if (typeof this.state.formula == 'string') {
            const functionMatches = (_a = this.state.formula) === null || _a === void 0 ? void 0 : _a.match(docBoxRe);
            if (!functionMatches) {
                return undefined;
            }
            // We take the _last_ function that has been written, as this is the funciton
            // being edited currently.
            const lastFunction = functionMatches[functionMatches.length - 1];
            // Strip off the last ( from the function name
            const lastFunctionClean = lastFunction.substring(0, lastFunction.length - 1).toUpperCase();
            return functionDocumentationObjects.find((funcDocObject) => {
                return funcDocObject.function === lastFunctionClean;
            });
        }
        return undefined;
    }
    // If you mouseover a suggestion, this selects it
    onMouseEnterSuggestion(suggestionIndex) {
        this.setState({ suggestionIndex: suggestionIndex });
    }
    selectSuggestion(type, fullSuggestion, endingString) {
        // If the editor is in SINGLE_CELL_EDITING mode, don't let the user select any suggestions
        if (this.state.mode === CellEditingMode.SINGLE_CELL_EDITING) {
            return undefined;
        }
        // We fill in the rest of the suggestion, and reset the suggestion box
        const stripped = this.state.formula.substring(0, this.state.formula.length - endingString.length);
        let newFormula;
        if (type === 'function') {
            // Upper case, add paren, if a function
            newFormula = stripped + fullSuggestion.toUpperCase() + '(';
        }
        else {
            // Add column header, if column header
            newFormula = stripped + fullSuggestion;
        }
        this.updateFormulaState(newFormula);
        this.setState({
            suggestionIndex: 0
        }, () => {
            var _a;
            // TODO: add documentation about when this runs... (A close?)
            // We also refocus on the input feild
            (_a = this.state.input) === null || _a === void 0 ? void 0 : _a.focus();
            // And then make sure the cursor index is at the right location
            this.updateCursorIndex(0);
        });
    }
    render() {
        const suggestions = this.getCurrentSuggestions();
        const documentationBoxFunction = this.getDocumentationBoxFunction();
        return (React.createElement("div", null,
            React.createElement("input", { ref: (input) => {
                    if (!this.state.input) {
                        this.setState({ input: input }, () => {
                            var _a;
                            // See note in afterGuiAttached. This is a workaround that makes sure 
                            // the input is always focused on after it is displayed to the user. 
                            // Possibly related: https://stackoverflow.com/questions/44074747/componentdidmount-called-before-ref-callback
                            (_a = this.state.input) === null || _a === void 0 ? void 0 : _a.focus();
                        });
                    }
                }, className: "mito-cell-editor ag-cell-inline-editing", name: "value", autoComplete: "off", value: this.state.formula, onChange: this.handleOnChange, onKeyDown: this.onKeyDown, onClick: this.onClick, tabIndex: 1 }),
            suggestions !== undefined && this.props.isFormulaColumn &&
                React.createElement(SuggestionBox, { suggestions: suggestions, index: this.state.suggestionIndex, onMouseEnterSuggestion: this.onMouseEnterSuggestion, onSelectSuggestion: this.selectSuggestion }),
            suggestions === undefined && documentationBoxFunction && this.props.isFormulaColumn &&
                React.createElement(DocumentationBox, { funcDocObject: documentationBoxFunction })));
    }
}
//# sourceMappingURL=MitoCellEditor.js.map