// Copyright (c) Mito
import fscreen from 'fscreen';
import React, { useCallback, useEffect, useRef, useState } from 'react';
import ReactDOM from 'react-dom';
import '../../../css/elements/Dropdown.css';
import { useCallOnAnyClick } from '../../hooks/useCallOnAnyClick';
import { classNames } from '../../utils/classNames';
// NOTE: these must match their definitions in the Dropdown.css
const MAX_HEIGHT = 300;
const MAX_WIDTH = 400;
/*
    Helper functions for figuring out where to place the dropdown
    so that it is not out of bounds of the screen.

    To do so, each function uses the MAX_HEIGHT and MAX_WIDTH variables
    above and checks that in the worst case, the entire dropdown will
    still be on screen given a certain bounding rect setting!
*/
const topInBounds = (top) => {
    const windowHeight = window.innerHeight;
    return top + MAX_HEIGHT < windowHeight;
};
const leftInBounds = (left) => {
    const widthWidth = window.innerWidth;
    return left + MAX_WIDTH < widthWidth;
};
const bottomInBounds = (bottom) => {
    return bottom - MAX_HEIGHT > 0;
};
const rightInBounds = (right) => {
    return right - MAX_WIDTH > 0;
};
/**
 * The Dropdown component is used to display dropdown menus. It closes
 * automatically when the user clicks anywhere, including on the items
 * in the dropdown itself.
 *
 * The dropdown component places itself on top of everything else on screen,
 * and will place itself near its parent component. It will make sure that all
 * the DropdownItems it is displaying are on screen.
 */
const Dropdown = (props) => {
    // Close the dropdown anytime anyone clicks anywhere
    useCallOnAnyClick(props.closeDropdown);
    // Store some information about where we want the bounding rect of the dropdown
    // so that we can place it in the right location
    const [boundingRect, setBoundingRect] = useState({
        top: undefined,
        bottom: undefined,
        right: undefined,
        left: undefined
    });
    // The anchor for the dropdown, that is placed as a child the parent
    // and is how we figure out who the parent of the dropdown is
    const dropdownAnchor = useRef(null);
    // The div that contains the actual dropdown, and is added to the highest
    // level of the html, so the dropdown's z-index can be above everything else
    const dropdownContainerElement = useRef(document.createElement("div"));
    // We store if this select was created when the element was in fullscreen or
    // not, which is helpful for cleaning up the select
    const [isNotFullscreen] = useState(fscreen.fullscreenElement === undefined || fscreen.fullscreenElement === null);
    // If we are in fullscreen, we need the closest mito container, and so
    // we store it in this case so we have access to it
    const mitoContainerRef = useRef(null);
    // This just cleans up the dropdown container, when the component unrenders
    // So that we don't leave a bunch of divs lying around
    useEffect(() => {
        return () => {
            if (isNotFullscreen) {
                document.body.removeChild(dropdownContainerElement.current);
            }
            else {
                if (mitoContainerRef.current) {
                    mitoContainerRef.current.removeChild(dropdownContainerElement.current);
                }
            }
        };
    }, []);
    // When we first render the dropdownAnchor, make sure to save it 
    // and to update the position of the dropdown
    const setRef = useCallback((unsavedDropdownAnchor) => {
        if (unsavedDropdownAnchor !== null) {
            // Save this node, so that we can update 
            dropdownAnchor.current = unsavedDropdownAnchor;
            if (isNotFullscreen) {
                // Add element to the document, if we're not in 
                // fullscreen mode
                document.body.append(dropdownContainerElement.current);
            }
            else {
                // If we are in fullscreen mode, then place the dropdownContainerElement in the
                // mitoContainer so that it is visible in fullscreen mode
                const mitoContainer = unsavedDropdownAnchor.closest('.mito-container');
                if (mitoContainer) {
                    mitoContainer.appendChild(dropdownContainerElement.current);
                    mitoContainerRef.current = mitoContainer;
                }
            }
            updateDropdownPosition(unsavedDropdownAnchor);
        }
    }, []);
    // Refresh the location of the dropdown for as long as it's open
    // NOTE: we could use an onscroll listener, but in practice it's 
    // wayyyy too laggy. So no thanks.
    useEffect(() => {
        const interval = setInterval(() => {
            if (dropdownAnchor.current !== null) {
                updateDropdownPosition(dropdownAnchor.current);
            }
        }, 25);
        return () => clearInterval(interval);
    }, []);
    /*
        This function actually does the work of figuring out where the
        dropdown should be placed. To do so, it takes the dropdownContainer
        and finds it's parent, which is effectively the parent of the dropdown.

        Then, it figures out where it can place the dropdown in a way that doesn't
        go offscreen and can actually be seen by the user. It tries the following
        arrangements:

        1. Check if (bottom, left) of the parent can be the (top, left) of the dropdown.
        2. Check if (bottom, right) of the parent can be the (top, right) of the dropdown.
        3. Check if (top, left) of the parent can be the (bottom, left) of the dropdown.
        4. If none of the above work, set (top, right) to the (bottom, right) of the dropdown.
    */
    const updateDropdownPosition = (dropdownContainer) => {
        const parentElement = dropdownContainer.parentElement || dropdownContainer;
        const parentBoundingClientRect = parentElement.getBoundingClientRect();
        const parentTop = parentBoundingClientRect.top;
        const parentBottom = parentBoundingClientRect.bottom;
        const parentLeft = parentBoundingClientRect.left;
        const parentRight = parentBoundingClientRect.right;
        if (topInBounds(parentBottom) && leftInBounds(parentLeft)) {
            setBoundingRect({
                top: parentBottom,
                bottom: undefined,
                right: undefined,
                left: parentLeft
            });
        }
        else if (topInBounds(parentBottom) && rightInBounds(parentRight)) {
            setBoundingRect({
                top: parentBottom,
                bottom: undefined,
                right: window.innerWidth - parentRight,
                left: undefined
            });
        }
        else if (bottomInBounds(parentTop) && leftInBounds(parentLeft)) {
            setBoundingRect({
                top: undefined,
                bottom: window.innerHeight - parentTop,
                right: undefined,
                left: parentLeft
            });
        }
        else {
            setBoundingRect({
                top: undefined,
                bottom: window.innerHeight - parentTop,
                right: window.innerWidth - parentRight,
                left: undefined
            });
        }
    };
    // If there are more than 4 elements in the dropdown, compress it so that the user
    // can see more of it
    const compressedClass = React.Children.count(props.children) > 4 ? 'dropdown-compressed' : '';
    return (React.createElement("div", { ref: setRef }, ReactDOM.createPortal(React.createElement("div", { className: classNames('dropdown', compressedClass), style: {
            position: 'absolute',
            top: boundingRect.top,
            bottom: boundingRect.bottom,
            right: boundingRect.right,
            left: boundingRect.left,
        } }, props.children), dropdownContainerElement.current)));
};
export default Dropdown;
//# sourceMappingURL=Dropdown.js.map