// Copyright (c) Mito
import React from 'react';
// import css
import "../../../../css/taskpanes/ControlPanel/ControlPanelTaskpane.css";
import { isFilterGroup } from './FilterAndSortTab/filter/filterTypes';
import FilterCard from './FilterAndSortTab/filter/FilterCard';
import SortCard from './FilterAndSortTab/SortCard';
import ColumnSummaryGraph from './SummaryStatsTab/ColumnSummaryGraph';
import ColumnNameCard from './ColumnNameCard';
import ColumnSummaryStatistics from './SummaryStatsTab/ColumnSummaryStatistics';
import ControlPanelTaskpaneTabs from './ControlPanelTaskpaneTabs';
import DtypeCard from './FilterAndSortTab/DtypeCard';
import { ValuesTab } from './ValuesTab/ValuesTab';
import { areFiltersEqual, getEmptyFilterData, getExclusiveFilterData, isValidFilter, parseFilter } from './FilterAndSortTab/filter/utils';
/*
    We wait 500ms before sending a filter message to make sure
    that as the user is typing key changes, we don't queue up a
    ton of filtering messages.
*/
const FILTER_MESSAGE_DELAY = 500;
export var ControlPanelTab;
(function (ControlPanelTab) {
    ControlPanelTab["FilterSort"] = "filter/sort";
    ControlPanelTab["UniqueValues"] = "unqiue values";
    ControlPanelTab["SummaryStats"] = "summary stats";
})(ControlPanelTab || (ControlPanelTab = {}));
/*
    A modal that allows a user to rename, sort, and filter a column.
*/
class ControlPanelTaskpane extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            filters: props.filters,
            operator: props.operator,
            stepID: '',
            timeoutId: undefined
        };
        this.sendFilterUpdateMessage = this.sendFilterUpdateMessage.bind(this);
        this._sendFilterUpdateMessage = this._sendFilterUpdateMessage.bind(this);
        this.addFilter = this.addFilter.bind(this);
        this.addFilterGroup = this.addFilterGroup.bind(this);
        this.addFilterToGroup = this.addFilterToGroup.bind(this);
        this.deleteFilter = this.deleteFilter.bind(this);
        this.deleteFilterFromGroup = this.deleteFilterFromGroup.bind(this);
        this.setFilter = this.setFilter.bind(this);
        this.setFilterInGroup = this.setFilterInGroup.bind(this);
        this.setOperator = this.setOperator.bind(this);
        this.setOperatorInGroup = this.setOperatorInGroup.bind(this);
        this.toggleExclusiveFilter = this.toggleExclusiveFilter.bind(this);
    }
    /*
        We only send a filter message out after .5 seconds, and delete any
        message that has not been sent if a new one comes in. This makes sure we don't
        have a huge backlog of filter message waiting to be processed
    */
    async sendFilterUpdateMessage() {
        if (this.state.timeoutId) {
            global.clearTimeout(this.state.timeoutId);
        }
        const newTimeoutId = global.setTimeout(this._sendFilterUpdateMessage, FILTER_MESSAGE_DELAY);
        this.setState({ timeoutId: newTimeoutId });
    }
    /*
        NOTE: only call this through the sendFilterUpdateMessage function, to make sure
        buffering messages works properly.

        Before sending the displayed filters, we parse all the number filters from strings
        to numbers, and then filter out all of the invalid filters (as to not cause errors)
    */
    async _sendFilterUpdateMessage() {
        // To handle decimals, we allow decimals to be submitted, and then just
        // parse them before they are sent to the back-end
        const parsedFilters = this.state.filters.map((filterOrGroup) => {
            if (isFilterGroup(filterOrGroup)) {
                return {
                    filters: filterOrGroup.filters.map((filter) => {
                        return parseFilter(filter);
                    }),
                    operator: filterOrGroup.operator
                };
            }
            else {
                return parseFilter(filterOrGroup);
            }
        });
        const filtersToApply = parsedFilters.map((filterOrGroup) => {
            // Filter out these incomplete filters from the group
            if (isFilterGroup(filterOrGroup)) {
                return {
                    filters: filterOrGroup.filters.filter((filter) => {
                        return isValidFilter(filter);
                    }),
                    operator: filterOrGroup.operator
                };
            }
            else {
                return filterOrGroup;
            }
        }).filter((filterOrGroup) => {
            // Filter out the groups if they have no valid filters in them
            if (isFilterGroup(filterOrGroup)) {
                return filterOrGroup.filters.length > 0;
            }
            // And then we filter the non group filters to be non-empty
            return isValidFilter(filterOrGroup);
        });
        const stepID = await this.props.mitoAPI.sendFilterMessage(this.props.selectedSheetIndex, this.props.columnID, filtersToApply, this.state.operator, this.state.stepID, this.props.tab);
        this.setState({ stepID: stepID });
    }
    /*
        Adds a new, blank filter to the end of the filters list
    */
    addFilter() {
        this.setState(prevState => {
            const newFilters = [...prevState.filters];
            newFilters.push(getEmptyFilterData(this.props.columnMitoType));
            return {
                filters: newFilters
            };
        }, () => {
            void this.sendFilterUpdateMessage();
        });
    }
    /*
        Creates a new filter group (at the bottom) with a single empty
        filter
    */
    addFilterGroup() {
        this.setState(prevState => {
            const newFilters = [...prevState.filters];
            newFilters.push({
                filters: [
                    getEmptyFilterData(this.props.columnMitoType)
                ],
                operator: 'And'
            });
            return {
                filters: newFilters
            };
        }, () => {
            void this.sendFilterUpdateMessage();
        });
    }
    /*
        Adds a blank new filter to the end of a specific group
    */
    addFilterToGroup(groupIndex) {
        this.setState(prevState => {
            const newFilters = [...prevState.filters];
            const filterGroup = newFilters[groupIndex];
            if (isFilterGroup(filterGroup)) {
                // If we do have a filter group at that groupIndex, then we add a new filter to it
                filterGroup.filters.push(getEmptyFilterData(this.props.columnMitoType));
                return {
                    filters: newFilters
                };
            }
            else {
                // We make no changes if this was not a filter group, which should never occur
                return null;
            }
        }, () => {
            void this.sendFilterUpdateMessage();
        });
    }
    /*
        Toggles the exclusive filter for a specific value. An exclusive filter is the NOT_EXACTLY filter condition
        for strings, numbers, and dates. For booleans its either the IS_TRUE or IS_FALSE conditions.

        For a specific value in this column, the toggleExclusiveFilter determines if there is an exclusive filter that is excluding the value.
        If there is, it removes it. If there's is not, it applies it.
    */
    toggleExclusiveFilter(value) {
        // Generate the filter
        const exclusiveFilter = getExclusiveFilterData(this.props.columnMitoType, value);
        this.setState(prevState => {
            let newFilters = [...prevState.filters];
            const originalFilterLength = newFilters.length;
            // Remove the filter if it exists
            newFilters = newFilters.filter(filter => {
                return isFilterGroup(filter) || !areFiltersEqual(filter, exclusiveFilter);
            });
            // If the filter didn't exist, then add it. 
            if (newFilters.length === originalFilterLength) {
                newFilters.push(exclusiveFilter);
            }
            return {
                filters: newFilters
            };
        }, () => {
            void this.sendFilterUpdateMessage();
        });
    }
    /*
        Deletes a filter that is at the given index in the main
        filter list.
    */
    deleteFilter(filterIndex) {
        this.setState(prevState => {
            const newFilters = [...prevState.filters];
            newFilters.splice(filterIndex, 1);
            return {
                filters: newFilters
            };
        }, () => {
            void this.sendFilterUpdateMessage();
        });
    }
    /*
        Deletes a filter that is at a given index in filter list
        of a specific filter group
    */
    deleteFilterFromGroup(groupIndex, filterIndex) {
        this.setState(prevState => {
            const newFilters = [...prevState.filters];
            const filterGroup = newFilters[groupIndex];
            if (isFilterGroup(filterGroup)) {
                // If we do have a filter group at that groupIndex, then we delete the filter
                // at the passed filterIndex
                filterGroup.filters.splice(filterIndex, 1);
                // If there are no filters left in this group, then we remove the entire group
                if (filterGroup.filters.length === 0) {
                    newFilters.splice(groupIndex, 1);
                }
                return {
                    filters: newFilters
                };
            }
            else {
                // We make no changes if this was not a filter group, which should never occur
                return null;
            }
        }, () => {
            void this.sendFilterUpdateMessage();
        });
    }
    /*
        Sets a filter at the given index to the new filter value
    */
    setFilter(filterIndex, filter) {
        this.setState(prevState => {
            const newFilters = [...prevState.filters];
            newFilters[filterIndex] = filter;
            return {
                filters: newFilters
            };
        }, () => {
            void this.sendFilterUpdateMessage();
        });
    }
    /*
        Sets a filter at the given filterIndex in the specific group at the given
        groupIndex to the new filter value
    */
    setFilterInGroup(groupIndex, filterIndex, filter) {
        this.setState(prevState => {
            const newFilters = [...prevState.filters];
            const filterGroup = newFilters[groupIndex];
            if (isFilterGroup(filterGroup)) {
                filterGroup.filters[filterIndex] = filter;
                return {
                    filters: newFilters
                };
            }
            else {
                // We make no changes if this was not a filter group, which should never occur
                return null;
            }
        }, () => {
            void this.sendFilterUpdateMessage();
        });
    }
    /*
        Sets the operator that combines the highest level filters and
        filter groups
    */
    setOperator(operator) {
        this.setState({ operator: operator }, () => {
            void this.sendFilterUpdateMessage();
        });
    }
    /*
        Sets the operator that combines a specific filter group
    */
    setOperatorInGroup(groupIndex, operator) {
        this.setState(prevState => {
            const newFilters = [...prevState.filters];
            const filterGroup = newFilters[groupIndex];
            if (isFilterGroup(filterGroup)) {
                filterGroup.operator = operator;
                return {
                    filters: newFilters
                };
            }
            else {
                // We make no changes if this was not a filter group, which should never occur
                return null;
            }
        }, () => {
            void this.sendFilterUpdateMessage();
        });
    }
    render() {
        return (React.createElement(React.Fragment, null,
            React.createElement("div", { className: 'control-panel-taskpane-container default-taskpane-div' },
                React.createElement(ColumnNameCard, { columnID: this.props.columnID, columnIDsMapArray: this.props.columnIDsMapArray, openEditingColumnHeader: this.props.openEditingColumnHeader, selectedSheetIndex: this.props.selectedSheetIndex, mitoAPI: this.props.mitoAPI, setCurrOpenTaskpane: this.props.setCurrOpenTaskpane, setColumnRenameOpenOrClosed: this.props.setColumnRenameOpenOrClosed }),
                React.createElement("div", { className: 'default-taskpane-body-div mb-30px' },
                    this.props.tab === ControlPanelTab.FilterSort &&
                        React.createElement(React.Fragment, null,
                            React.createElement(DtypeCard, { selectedSheetIndex: this.props.selectedSheetIndex, columnID: this.props.columnID, columnFormula: this.props.columnFormula, mitoAPI: this.props.mitoAPI, lastStepIndex: this.props.lastStepIndex }),
                            React.createElement(SortCard, { selectedSheetIndex: this.props.selectedSheetIndex, columnID: this.props.columnID, mitoAPI: this.props.mitoAPI }),
                            React.createElement(FilterCard, { selectedSheetIndex: this.props.selectedSheetIndex, columnID: this.props.columnID, filters: this.state.filters, columnMitoType: this.props.columnMitoType, operator: this.props.operator, mitoAPI: this.props.mitoAPI, setOperator: this.setOperator, setFilterInGroup: this.setFilterInGroup, setOperatorInGroup: this.setOperatorInGroup, deleteFilterFromGroup: this.deleteFilterFromGroup, addFilterToGroup: this.addFilterToGroup, setFilter: this.setFilter, deleteFilter: this.deleteFilter, addFilter: this.addFilter, addFilterGroup: this.addFilterGroup })),
                    this.props.tab === ControlPanelTab.UniqueValues &&
                        React.createElement(React.Fragment, null,
                            React.createElement(ValuesTab, { selectedSheetIndex: this.props.selectedSheetIndex, columnID: this.props.columnID, mitoAPI: this.props.mitoAPI, filters: this.props.filters, columnMitoType: this.props.columnMitoType, toggleExclusiveFilter: this.toggleExclusiveFilter })),
                    this.props.tab === ControlPanelTab.SummaryStats &&
                        React.createElement(React.Fragment, null,
                            React.createElement(ColumnSummaryGraph, { selectedSheetIndex: this.props.selectedSheetIndex, columnID: this.props.columnID, mitoAPI: this.props.mitoAPI }),
                            React.createElement(ColumnSummaryStatistics, { selectedSheetIndex: this.props.selectedSheetIndex, columnID: this.props.columnID, mitoAPI: this.props.mitoAPI })))),
            React.createElement(ControlPanelTaskpaneTabs, { selectedTab: this.props.tab, setSelectedColumnControlPanelTab: this.props.setSelectedColumnControlPanelTab })));
    }
}
export default ControlPanelTaskpane;
//# sourceMappingURL=ControlPanelTaskpane.js.map