// Copyright (c) Mito
import React, { useRef, useState } from "react";
import { Fragment } from "react";
import Dropdown from "../../elements/Dropdown";
import DropdownItem from "../../elements/DropdownItem";
import Input from "../../elements/Input";
import DropdownIcon from "../../icons/DropdownIcon";
import { ModalEnum } from "../../modals/modals";
/*
    Helper function for validating saved analysis renames. Valid renames are:
    - not duplicated names
    - not empty
*/
const getAnalysisNameError = (savedAnalysisNames, newAnalysisName) => {
    if (newAnalysisName === '') {
        return 'The analysis cannot be empty.';
    }
    else if (savedAnalysisNames.includes(newAnalysisName)) {
        return 'That name is taken by another analysis.';
    }
    else {
        return '';
    }
};
const SavedAnalysisCard = (props) => {
    const [isRename, setIsRename] = useState(false);
    const [newSavedAnalysisName, setNewSavedAnalysisName] = useState(props.savedAnalysisName);
    const [renameError, setRenameError] = useState('');
    const [savedAnalysisDescription, setSavedAnalysisDescription] = useState(undefined);
    const elementRef = useRef(null);
    // This function applies the saved analysis
    const applySavedAnalysis = async () => {
        // First, we check if there are any simple import steps that we need to handle with special
        // care, specifically by asking the users to switch out the files for new files.
        const importSummary = await props.mitoAPI.getImportSummary(props.savedAnalysisName);
        if (Object.keys(importSummary).length !== 0) {
            props.setModal({ type: ModalEnum.ReplayImport, analysisName: props.savedAnalysisName, importSummary: importSummary });
            return;
        }
        await props.mitoAPI.sendUseExistingAnalysisUpdateMessage(props.savedAnalysisName);
    };
    // This function deletes the saved analysis
    const deleteSavedAnalysis = async () => {
        props.setSelectedAnalysisAndDescription(undefined, undefined);
        await props.mitoAPI.sendDeleteSavedAnalysisMessage(props.savedAnalysisName);
    };
    // This function renames the saved analysis after making sure the name is valid
    const renameSavedAnalysis = async () => {
        // If the new name is the same as the old name, turn off the error and close renaming
        if (props.savedAnalysisName === newSavedAnalysisName) {
            setRenameError('');
            setIsRename(false);
            return;
        }
        // Validate the rename
        const renameAnalysisError = getAnalysisNameError(props.savedAnalysisNames, newSavedAnalysisName);
        setRenameError(renameAnalysisError);
        // If the new name is valid, send rename event
        if (renameAnalysisError === '') {
            await props.mitoAPI.sendRenameSavedAnalysisMessage(props.savedAnalysisName, newSavedAnalysisName);
            props.setSelectedAnalysisAndDescription(newSavedAnalysisName, savedAnalysisDescription);
            setIsRename(false);
        }
    };
    // This function receives the steps of the saved analysis to be displayed by the taskpane
    const getSavedAnalysisDescription = async () => {
        const savedAnalysisDescription = await props.mitoAPI.getSavedAnalysisDescription(props.savedAnalysisName);
        setSavedAnalysisDescription(savedAnalysisDescription);
        props.setSelectedAnalysisAndDescription(newSavedAnalysisName, savedAnalysisDescription);
        props.setOpenedDropdown(undefined);
        /*
            We use a log event here, but not in the other functions, because the other functions
            are update events that are automatically logged. This function goes through api.py and therefore
            needs manual logging.
        */
        void props.mitoAPI.sendLogMessage('clicked_saved_analysis_name_log_event', {
            saved_analysis_name: props.savedAnalysisName,
        });
    };
    return (React.createElement(Fragment, null,
        React.createElement("div", { ref: elementRef },
            React.createElement("div", { className: 'saved-analysis-card' },
                React.createElement("div", { className: 'saved-analysis-card-analysis-name', onClick: () => { void getSavedAnalysisDescription(); }, onDoubleClick: () => { setIsRename(true); } },
                    isRename &&
                        React.createElement("form", { onSubmit: async (e) => { e.preventDefault(); await renameSavedAnalysis(); }, onBlur: renameSavedAnalysis },
                            React.createElement(Input, { value: newSavedAnalysisName, onChange: (e) => { setNewSavedAnalysisName(e.target.value); }, autoFocus: true })),
                    !isRename &&
                        props.savedAnalysisName),
                React.createElement("div", null,
                    React.createElement("div", { className: 'saved-analysis-card-dropdown-div', onClick: () => {
                            /*
                                If the saved analysis card that is clicked is the analysis card in the summary section
                                of the taskpane, mark it using '-described-analysis', so that we can determine whether the
                                user clicked on the Dropdown that is part of the list of saved analyses or if it was
                                the analysis card used in the summary section.

                                We do this so only one dropdown opens instead of the one in the list and the summary section.
                            */
                            if (props.isDescribedAnalysis) {
                                props.setOpenedDropdown(props.savedAnalysisName + '-described-analysis');
                            }
                            else {
                                props.setOpenedDropdown(props.savedAnalysisName);
                            }
                        } },
                        React.createElement(DropdownIcon, null)),
                    props.openedDropdown &&
                        React.createElement(Dropdown, { closeDropdown: () => { props.setOpenedDropdown(undefined); } },
                            React.createElement(DropdownItem, { title: 'Apply Analysis', onClick: applySavedAnalysis }),
                            React.createElement(DropdownItem, { title: 'Rename Analysis', onClick: () => void setIsRename(true) }),
                            React.createElement(DropdownItem, { title: 'Delete Analysis', onClick: deleteSavedAnalysis })))),
            renameError !== '' &&
                React.createElement("p", { className: 'replay-analysis-rename-error' }, renameError))));
};
export default SavedAnalysisCard;
//# sourceMappingURL=SavedAnalysisCard.js.map