// Copyright (c) Mito
import React, { useState } from 'react';
import fscreen from 'fscreen';
// Import CSS
import "../../../css/toolbar.css";
import { ModalEnum } from '../modals/modals';
// Import Components 
import { TaskpaneType } from '../taskpanes/taskpanes';
import ToolbarButton from './ToolbarButton';
import { ToolbarButtonType } from './utils';
import { DFSource } from '../../types/DFSources';
import { CellEditingMode } from '../MitoCellEditor';
const Toolbar = (props) => {
    /* Adds a new column directly to the right of the selected column, with A, B, C... as the name */
    const addColumn = () => {
        // We turn off editing mode, if it is on
        props.setEditingMode(CellEditingMode.OFF, "", -1);
        // we close the editing taskpane if its open
        props.closeOpenEditingPopups();
        /*
        * Helper function that, given a number, returns Excel column that corresponds to this number (1-indexed)
        */
        function toColumnName(num) {
            let ret;
            let a = 1;
            let b = 26;
            for (ret = ''; (num -= a) >= 0; a = b, b *= 26) {
                ret = String.fromCharCode(Math.round((num % b) / a) + 65) + ret;
            }
            return ret;
        }
        let inc = 1;
        let newColumnHeader = toColumnName(props.sheetJSON.columnIDs.length + inc);
        // If the column header is in the sheet already, we bump and keep trying
        const columnHeaders = Object.values(props.columnIDMap);
        while (columnHeaders.includes(newColumnHeader)) {
            inc++;
            newColumnHeader = toColumnName(props.sheetJSON.columnIDs.length + inc);
        }
        // The new column should be placed 1 position to the right of the selected column
        const newColumnHeaderIndex = props.sheetJSON.columnIDs.findIndex((columnID) => columnID === props.selectedColumnID) + 1;
        void props.mitoAPI.sendColumnAddMessage(props.selectedSheetIndex, newColumnHeader, newColumnHeaderIndex);
    };
    /* Undoes the last step that was created */
    const undo = () => {
        // We turn off editing mode, if it is on
        props.setEditingMode(CellEditingMode.OFF, "", -1);
        // we close the editing taskpane if its open
        props.closeOpenEditingPopups([TaskpaneType.PIVOT]);
        void props.mitoAPI.sendUndoMessage();
    };
    /* Redoes the last step that was undo, if it exists */
    const redo = () => {
        // We turn off editing mode, if it is on
        props.setEditingMode(CellEditingMode.OFF, "", -1);
        // we close the editing taskpane if its open
        props.closeOpenEditingPopups([TaskpaneType.PIVOT]);
        void props.mitoAPI.sendRedoMessage();
    };
    const openClearAnalysisModal = () => {
        // Log that we opened this
        void props.mitoAPI.sendLogMessage('click_open_reset_analysis');
        // We turn off editing mode, if it is on
        props.setEditingMode(CellEditingMode.OFF, "", -1);
        // we close the editing taskpane if its open
        props.closeOpenEditingPopups();
        // Close all taskpanes if they are open, to make sure the state is not out of sync
        props.setCurrOpenTaskpane({ type: TaskpaneType.NONE });
        props.setModal({ type: ModalEnum.ClearAnalysis });
    };
    /* Saves the current file as as an exported analysis */
    const downloadAnalysis = () => {
        // We turn off editing mode, if it is on
        props.setEditingMode(CellEditingMode.OFF, "", -1);
        // we close the editing taskpane if its open
        props.closeOpenEditingPopups();
        props.setCurrOpenTaskpane({ type: TaskpaneType.DOWNLOAD });
    };
    const [fullscreen, setFullscreen] = useState(false);
    fscreen.onfullscreenchange = () => {
        setFullscreen(!!fscreen.fullscreenElement);
        void props.mitoAPI.sendLogMessage('button_toggle_fullscreen', {
            // Note that this is true when _end_ in fullscreen mode, and 
            // false when we _end_ not in fullscreen mode, which is much
            // more natural than the alternative
            fullscreen: !!fscreen.fullscreenElement
        });
    };
    /*
        Toggles if Mito is full screen or not
    */
    const toggleFullscreen = () => {
        // We toggle to the opposite of whatever the fullscreen actually is (as detected by the
        // fscreen library), and then we set the fullscreen state variable to that state (in the callback
        // above), so that the component rerenders propery
        const isNotFullscreen = fscreen.fullscreenElement === undefined || fscreen.fullscreenElement === null;
        if (isNotFullscreen && props.mitoContainerRef) {
            fscreen.requestFullscreen(props.mitoContainerRef);
        }
        else {
            fscreen.exitFullscreen();
        }
    };
    const openDocumentation = () => {
        // We turn off editing mode, if it is on
        props.setEditingMode(CellEditingMode.OFF, "", -1);
        // We log the opening of the documentation taskpane
        void props.mitoAPI.sendLogMessage('button_documentation_log_event', {
            stage: 'opened'
        });
        // we open the documentation taskpane
        props.setCurrOpenTaskpane({ type: TaskpaneType.DOCUMENTATION });
    };
    const openMerge = () => {
        // We turn off editing mode, if it is on
        props.setEditingMode(CellEditingMode.OFF, "", -1);
        // We open the merge taskpane
        props.setCurrOpenTaskpane({ type: TaskpaneType.MERGE });
    };
    const openPivotTable = async () => {
        // We turn off editing mode, if it is on
        props.setEditingMode(CellEditingMode.OFF, "", -1);
        // We check if the currently opened sheet is a result of a pivot table
        // and if so then we open the existing pivot table here, rather than
        // create a new pivot table. That is: if a user is on a pivot table, then
        // we let them edit that pivot table
        if (props.dfSources[props.selectedSheetIndex] === DFSource.Pivoted) {
            const existingPivotParams = await props.mitoAPI.getPivotParams(props.selectedSheetIndex);
            if (existingPivotParams !== undefined) {
                props.setCurrOpenTaskpane({
                    type: TaskpaneType.PIVOT,
                    destinationSheetIndex: props.selectedSheetIndex,
                    existingPivotParams: existingPivotParams
                });
            }
        }
        else {
            // Otherwise, just open a new pivot table
            props.setCurrOpenTaskpane({ type: TaskpaneType.PIVOT });
        }
    };
    const deleteColumn = async () => {
        // We turn off editing mode, if it is on
        props.setEditingMode(CellEditingMode.OFF, "", -1);
        // we close the editing taskpane if its open
        props.closeOpenEditingPopups();
        // And then delete the column that is up
        await props.mitoAPI.sendDeleteColumn(props.selectedSheetIndex, props.selectedColumnID);
    };
    const openSave = () => {
        // We turn off editing mode, if it is on
        props.setEditingMode(CellEditingMode.OFF, "", -1);
        // we close the editing taskpane if its open
        props.closeOpenEditingPopups();
        props.setCurrOpenTaskpane({ type: TaskpaneType.SAVE });
    };
    const openReplay = () => {
        // We turn off editing mode, if it is on
        props.setEditingMode(CellEditingMode.OFF, "", -1);
        // we close the editing taskpane if its open
        props.closeOpenEditingPopups();
        props.setCurrOpenTaskpane({ type: TaskpaneType.REPLAY_ANALYSIS });
    };
    const openImport = () => {
        // We turn off editing mode, if it is on
        props.setEditingMode(CellEditingMode.OFF, "", -1);
        // we close the editing taskpane if its open
        props.closeOpenEditingPopups();
        props.setCurrOpenTaskpane({ type: TaskpaneType.IMPORT });
    };
    const openGraph = () => {
        // We turn off editing mode, if it is on
        props.setEditingMode(CellEditingMode.OFF, "", -1);
        props.setCurrOpenTaskpane({ type: TaskpaneType.GRAPH, graphSidebarSheet: props.selectedSheetIndex });
    };
    const openSteps = () => {
        void props.mitoAPI.sendLogMessage('click_open_steps');
        props.setCurrOpenTaskpane({ type: TaskpaneType.STEPS });
    };
    const catchup = () => {
        // Fast forwards to the most recent step, allowing editing
        void props.mitoAPI.sendLogMessage('click_catch_up');
        void props.mitoAPI.checkoutStepByIndex(props.lastStepIndex);
    };
    return (React.createElement("div", { className: 'toolbar-container' },
        React.createElement("div", { className: 'toolbar-left-half' },
            React.createElement(ToolbarButton, { toolbarButtonType: ToolbarButtonType.UNDO, buttonTitle: "UNDO", onClick: undo }),
            React.createElement(ToolbarButton, { toolbarButtonType: ToolbarButtonType.REDO, buttonTitle: "REDO", onClick: redo }),
            React.createElement(ToolbarButton, { toolbarButtonType: ToolbarButtonType.CLEAR, buttonTitle: "CLEAR", onClick: openClearAnalysisModal }),
            React.createElement("div", { className: "toolbar-vertical-line" }),
            React.createElement(ToolbarButton, { toolbarButtonType: ToolbarButtonType.IMPORT, buttonTitle: "IMPORT", onClick: openImport }),
            React.createElement(ToolbarButton, { toolbarButtonType: ToolbarButtonType.EXPORT, buttonTitle: "EXPORT", onClick: downloadAnalysis }),
            React.createElement("div", { className: "toolbar-vertical-line" }),
            React.createElement(ToolbarButton, { toolbarButtonType: ToolbarButtonType.ADD_COL, buttonTitle: "ADD COL", onClick: addColumn, highlightToolbarButton: props.highlightAddColButton }),
            React.createElement(ToolbarButton, { toolbarButtonType: ToolbarButtonType.DEL_COL, buttonTitle: "DEL COL", onClick: deleteColumn }),
            React.createElement("div", { className: "toolbar-vertical-line" }),
            React.createElement(ToolbarButton, { toolbarButtonType: ToolbarButtonType.PIVOT, buttonTitle: "PIVOT", onClick: openPivotTable, highlightToolbarButton: props.highlightPivotTableButton }),
            React.createElement(ToolbarButton, { toolbarButtonType: ToolbarButtonType.MERGE, buttonTitle: "MERGE", onClick: openMerge }),
            React.createElement("div", { className: "toolbar-vertical-line" }),
            React.createElement(ToolbarButton, { toolbarButtonType: ToolbarButtonType.GRAPH, buttonTitle: "GRAPH", onClick: openGraph }),
            React.createElement("div", { className: "toolbar-vertical-line" }),
            React.createElement(ToolbarButton, { toolbarButtonType: ToolbarButtonType.SAVE, buttonTitle: "SAVE", onClick: openSave }),
            React.createElement(ToolbarButton, { toolbarButtonType: ToolbarButtonType.REPLAY, buttonTitle: "REPLAY", onClick: openReplay }),
            React.createElement("div", { className: "toolbar-vertical-line" })),
        React.createElement("div", { className: 'toolbar-right-half' },
            props.currStepIdx !== props.lastStepIndex &&
                React.createElement(ToolbarButton, { toolbarButtonType: ToolbarButtonType.CATCH_UP, buttonTitle: "CATCH UP", onClick: catchup }),
            React.createElement(ToolbarButton, { toolbarButtonType: ToolbarButtonType.STEPS, buttonTitle: "STEPS", onClick: openSteps }),
            React.createElement(ToolbarButton, { toolbarButtonType: ToolbarButtonType.DOCS, buttonTitle: "DOCS", onClick: openDocumentation }),
            React.createElement(ToolbarButton, { toolbarButtonType: fullscreen ? ToolbarButtonType.CLOSE_FULLSCREEN : ToolbarButtonType.OPEN_FULLSCREEN, buttonTitle: "FULLSCREEN", onClick: toggleFullscreen }))));
};
export default Toolbar;
//# sourceMappingURL=Toolbar.js.map