// Copyright (c) Mito
export const SHEET_HEIGHT_PIXELS = 400;
export const DEFAULT_COLUMN_WIDTH = 125;
/*
    Make sure that this is the width as suggestion-box-container
    in suggestion-box.css and the text editor in mitosheet.css
*/
export const LAST_COLUMN_DEFAULT_WIDTH = 300;
export const headerHeightParams = {
    defaultHeaderHeight: 40,
    charactersPerRow: 15,
    pixelsPerRow: 35 // number of pixels per row
};
/*
    A helper function for getting the width of columns, for all columns in all sheets. If given the prevColumnWidths, then
    will take columnWidths from there - and if not, will default to the DEFAULT_COLUMN_WIDTH.
*/
export function getColumnWidthsArray(sheetJSONArray, prevColumnWidthsArray) {
    // If there is a new sheet, this has obviously changed
    let changed = prevColumnWidthsArray === undefined || sheetJSONArray.length !== prevColumnWidthsArray.length;
    const newColumnWidthsArray = [];
    for (let i = 0; i < sheetJSONArray.length; i++) {
        const columnIDs = sheetJSONArray[i].columnIDs;
        const prevColumnWidths = prevColumnWidthsArray ? prevColumnWidthsArray[i] : undefined;
        const newColumnsWidthsResult = getColumnWidths(columnIDs, prevColumnWidths);
        newColumnWidthsArray.push(newColumnsWidthsResult.columnWidths);
        changed = changed || newColumnsWidthsResult.changed;
    }
    return {
        columnWidthsArray: newColumnWidthsArray,
        changed: changed
    };
}
/*
    A helper function for getting the width of a list of columns. If given the prevColumnWidths, then
    will take columnWidths from there - and if not, will default to the DEFAULT_COLUMN_WIDTH.

    Includes a helper variable, changed, which tells if you if anything is different between these objects.
*/
export function getColumnWidths(columnIDs, prevColumnWidths) {
    const newColumnWidths = {};
    let changed = prevColumnWidths === undefined;
    for (let i = 0; i < columnIDs.length; i++) {
        const columnHeader = columnIDs[i];
        if (prevColumnWidths && i === columnIDs.length - 1 && prevColumnWidths[columnHeader] < LAST_COLUMN_DEFAULT_WIDTH) {
            // If its the last column, make sure the column is at least LAST_COLUMN_DEFAULT_WIDTH so the cell editor is not cutoff
            newColumnWidths[columnHeader] = LAST_COLUMN_DEFAULT_WIDTH;
            changed = true;
        }
        else if (prevColumnWidths && i !== columnIDs.length - 1 && prevColumnWidths[columnHeader] === LAST_COLUMN_DEFAULT_WIDTH) {
            // If it was previously the last column and the user didn't change it from the default with, 
            // set the column width to DEFAULT_COLUMN_WIDTH.
            newColumnWidths[columnHeader] = DEFAULT_COLUMN_WIDTH;
            changed = true;
        }
        else if (prevColumnWidths && prevColumnWidths[columnHeader]) {
            // if the column has a width, then maintain it
            newColumnWidths[columnHeader] = prevColumnWidths[columnHeader];
        }
        else {
            newColumnWidths[columnHeader] = DEFAULT_COLUMN_WIDTH;
            // If there is a new column, then things have changed!
            changed = true;
        }
    }
    return {
        columnWidths: newColumnWidths,
        changed: changed
    };
}
export function calculateAndSetHeaderHeight(model_id) {
    var _a, _b;
    // we now do some work to find the best height to give the header row
    // first find the largest header
    let maxHeaderLength = 0;
    const gridApi = (_a = window.gridApiMap) === null || _a === void 0 ? void 0 : _a.get(model_id);
    // if the gridApi does not exist, stop
    if (gridApi === undefined) {
        return;
    }
    const rowNodeData = (_b = gridApi.getRowNode('0')) === null || _b === void 0 ? void 0 : _b.data;
    if (rowNodeData === undefined) {
        // This is undefined when the sheet is empty, and thus there aren't
        // any column headers to resize. Thus, we don't need to do anything.
        return;
    }
    Object.keys(rowNodeData).forEach((key) => {
        /*
            Because each row can only hold 15 characters on average,
            if any words are greater than 15 characters, it will be split into two rows.

            To make sure that we accommodate these special situations, we assume the worst case
            scenario, and add an extra entire line.
        */
        key = key.toString();
        let paddingCharacters = 0;
        getHeaderWords(key).forEach(word => {
            if (word.length > headerHeightParams.charactersPerRow) {
                paddingCharacters += headerHeightParams.charactersPerRow;
            }
        });
        maxHeaderLength = key.length + paddingCharacters > maxHeaderLength ? key.length + paddingCharacters : maxHeaderLength;
    });
    /*
        Then actually set the header height. We fit on average 15 letters
        per row with a starting width of 40 pixels and we give each row 30 px of space
    */
    gridApi.setHeaderHeight(Math.min(Math.max((maxHeaderLength / headerHeightParams.charactersPerRow - 1) * headerHeightParams.pixelsPerRow, headerHeightParams.defaultHeaderHeight), SHEET_HEIGHT_PIXELS / 4));
    gridApi.refreshHeader();
}
export function getHeaderWords(header) {
    /*
        We split the headers at _, and add _ to the latter half (e.g. first_name -> [first, _name]).

        If consecutive words can be combined and be less than the characters per row, then
        we combine them greedly.

    */
    const headerWords = [];
    const headerString = header.toString();
    const headerWordSplit = headerString.split(/(?=_)/g);
    let i;
    for (i = 0; i < headerWordSplit.length - 1; i++) {
        const firstWord = headerWordSplit[i];
        const secondWord = headerWordSplit[i + 1];
        if ((firstWord.length + secondWord.length) <= headerHeightParams.charactersPerRow) {
            headerWords.push(firstWord + secondWord);
            i++;
        }
        else {
            headerWords.push(firstWord);
        }
    }
    if (i === headerWordSplit.length - 1) {
        headerWords.push(headerWordSplit[headerWordSplit.length - 1]);
    }
    return headerWords;
}
//# sourceMappingURL=gridStyling.js.map