// Copyright (c) Mito
import { functionDocumentationObjects } from '../data/function_documentation';
/*
    Returns true iff the formula has a column header as a suffix.
*/
export function endsInColumnHeader(formula, columnHeaders) {
    for (let i = 0; i < columnHeaders.length; i++) {
        const columnHeaderString = columnHeaders[i].toString();
        if (formula.endsWith(columnHeaderString)) {
            return true;
        }
    }
    return false;
}
/*
    Returns a list of suggestions for column headers to append onto the given
    formula. These column headers much match some of the final characters in
    the formula. Returns undefined if there are no suggestions.
*/
export function getColumnHeaderSuggestions(formula, columnHeaders) {
    /* Remove the equals sign, so we don't match on it */
    if (formula.startsWith('=')) {
        formula = formula.substring(1);
    }
    // First, we check all the column headers, and if any of them are fully matched
    // on the end of the string, then we return nothing
    if (endsInColumnHeader(formula, columnHeaders)) {
        return undefined;
    }
    const matchingColumnHeaders = [];
    // We search the last 1, 2, 3, 4 and 5 characters for any partial match
    // with any column header, and terminate as soon as we find a column header 
    // that matches any of these substrings
    let endingColumnHeader = '';
    for (let numChars = 5; numChars > 0; numChars--) {
        endingColumnHeader = formula.substring(Math.max(0, formula.length - 1 - numChars));
        // Don't look for things that match an empty string
        if (endingColumnHeader === '') {
            continue;
        }
        for (let i = 0; i < columnHeaders.length; i++) {
            const columnHeaderString = columnHeaders[i].toString();
            if (columnHeaderString.indexOf(endingColumnHeader) > -1) {
                matchingColumnHeaders.push(columnHeaderString);
            }
        }
        if (matchingColumnHeaders.length > 0) {
            break;
        }
    }
    const suggestions = matchingColumnHeaders.map(columnHeader => {
        return {
            match: endingColumnHeader,
            suggestion: columnHeader,
            subtitle: 'A column in this sheet.',
            type: 'columnHeader'
        };
    });
    if (suggestions.length == 0) {
        return undefined;
    }
    return suggestions;
}
/*
    Gets the prefix of any function at the end of the formula. E.g.

    - func -> func
    - func123 -> undefined
    - function(newfunc -> newfunc
*/
export function getEndingFunction(formula) {
    /*
        Returns the maximal formula at the end of the formula.
    */
    let i = formula.length;
    for (i; i--; i >= 0) {
        const char = formula[i];
        if (!char.match(/[A-Za-z]+/g)) {
            // Stop as soon as we find a character that can't be in a function
            break;
        }
    }
    const possibleFunction = formula.substr(i + 1);
    return possibleFunction.length === 0 ? undefined : possibleFunction;
}
/*
    Returns all the function suggestions for the given formula, which
    it matches at the end of the formula. Case insensitive.
*/
export function getFunctionSuggestions(formula) {
    const endingFunction = getEndingFunction(formula);
    if (endingFunction === undefined) {
        return undefined;
    }
    const matchingFunctionDocObjects = functionDocumentationObjects.filter((funcDocObject) => {
        return funcDocObject.function.startsWith(endingFunction.toUpperCase());
    });
    const suggestions = matchingFunctionDocObjects.map(funcDocObject => {
        return {
            match: endingFunction,
            suggestion: funcDocObject.function,
            subtitle: funcDocObject.description,
            type: 'function'
        };
    });
    if (suggestions.length == 0) {
        return undefined;
    }
    return suggestions;
}
//# sourceMappingURL=suggestions.js.map